from datetime import datetime

from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_api.abc.transformer_abc import TransformerABC
from bot_api.model.auth_user_dto import AuthUserDTO
from bot_api.model.user_dto import UserDTO
from bot_data.model.auth_role_enum import AuthRoleEnum
from bot_data.model.auth_user import AuthUser
from bot_data.model.user import User
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class AuthUserTransformer(TransformerABC):
    @staticmethod
    def to_db(dto: AuthUserDTO) -> AuthUser:
        return AuthUser(
            dto.first_name,
            dto.last_name,
            dto.email,
            dto.password,
            None,
            None,
            None,
            None,
            None,
            datetime.now(),
            AuthRoleEnum.normal if dto.auth_role is None else AuthRoleEnum(dto.auth_role),
            auth_user_id=0 if dto.id is None else dto.id,
        )

    @staticmethod
    @ServiceProviderABC.inject
    def _is_technician(user: User, bot: DiscordBotServiceABC, permissions: PermissionServiceABC):
        guild = bot.get_guild(user.server.discord_id)
        member = guild.get_member(user.discord_id)
        return permissions.is_member_technician(member)

    @staticmethod
    @ServiceProviderABC.inject
    def _is_admin(user: User, bot: DiscordBotServiceABC, permissions: PermissionServiceABC):
        guild = bot.get_guild(user.server.discord_id)
        member = guild.get_member(user.discord_id)
        return permissions.is_member_admin(member)

    @staticmethod
    @ServiceProviderABC.inject
    def _is_moderator(user: User, bot: DiscordBotServiceABC, permissions: PermissionServiceABC):
        guild = bot.get_guild(user.server.discord_id)
        member = guild.get_member(user.discord_id)
        return permissions.is_member_moderator(member)

    @classmethod
    def to_dto(cls, db: AuthUser, password: str = None) -> AuthUserDTO:
        return AuthUserDTO(
            db.id,
            db.first_name,
            db.last_name,
            db.email,
            "" if password is None else password,
            db.confirmation_id,
            db.auth_role,
            List(
                UserDTO,
                db.users.select(
                    lambda u: UserDTO(
                        u.id,
                        u.discord_id,
                        u.xp,
                        u.server,
                        cls._is_technician(u),
                        cls._is_admin(u),
                        cls._is_moderator(u),
                    )
                ),
            ),
            db.created_at,
            db.modified_at,
        )
