from abc import ABC, abstractmethod

from cpl_query.extension import List

from bot_data.model.achievement import Achievement
from bot_data.model.user_got_achievement import UserGotAchievement


class AchievementRepositoryABC(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def get_achievements(self) -> List[Achievement]:
        pass

    @abstractmethod
    def get_achievement_by_id(self, id: int) -> Achievement:
        pass

    @abstractmethod
    def get_achievements_by_server_id(self, server_id: int) -> List[Achievement]:
        pass

    @abstractmethod
    def get_achievements_by_user_id(self, user_id: int) -> List[Achievement]:
        pass

    @abstractmethod
    def add_achievement(self, achievement: Achievement):
        pass

    @abstractmethod
    def update_achievement(self, achievement: Achievement):
        pass

    @abstractmethod
    def delete_achievement(self, achievement: Achievement):
        pass

    @abstractmethod
    def add_user_got_achievement(self, join: UserGotAchievement):
        pass

    @abstractmethod
    def delete_user_got_achievement(self, join: UserGotAchievement):
        pass
