from datetime import datetime
from typing import Optional

from cpl_core.database import TableABC

from bot_data.model.server import Server


class Achievement(TableABC):
    def __init__(
        self,
        name: str,
        attribute: str,
        operator: str,
        value: str,
        server: Optional[Server],
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._name = name
        self._attribute = attribute

        if operator not in ["==", "!=", "<=", ">=", "<", ">"]:
            raise ValueError("operator must be ==,!=,<=,>=,< or >")

        self._operator = operator
        self._value = value
        self._server = server

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value: str):
        self._name = value

    @property
    def attribute(self) -> str:
        return self._attribute

    @attribute.setter
    def attribute(self, value: str):
        self._attribute = value

    @property
    def operator(self) -> str:
        return self._operator

    @operator.setter
    def operator(self, value: str):
        self._operator = value

    @property
    def value(self) -> str:
        return self._value

    @value.setter
    def value(self, value: str):
        self._value = value

    @property
    def server(self) -> Server:
        return self._server

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `Achievements`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `Achievements`
            WHERE `Id` = {id};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `Achievements` (
                `Name`, `Attribute`, `Operator`, `Value`, `ServerId`
            ) VALUES (
                '{self._name}',
                '{self._attribute}',
                '{self._operator}',
                '{self._value}',
                {self._server.id}
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
                    UPDATE `Achievements`
                    SET `Name` = '{self._name}',
                    `Attribute` = '{self._attribute}',
                    `Operator` = '{self._operator}',
                    `Value` = '{self._value}'
                    WHERE `Id` = {self._id};
                """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `Achievements`
            WHERE `Id` = {self._id};
        """
        )
