from datetime import datetime
from typing import Optional

from cpl_core.database import TableABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.server import Server


class AutoRole(TableABC):
    def __init__(
        self,
        server: Optional[Server],
        channel_id: int,
        dc_message_id: int,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._auto_role_id = id
        self._server = server
        self._discord_channel_id = channel_id
        self._discord_message_id = dc_message_id

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._auto_role_id

    @property
    def server(self) -> Server:
        return self._server

    @property
    def discord_channel_id(self) -> int:
        return self._discord_channel_id

    @discord_channel_id.setter
    def discord_channel_id(self, value: int):
        self._discord_channel_id = value

    @property
    @ServiceProviderABC.inject
    def discord_channel_name(self, bot: DiscordBotServiceABC) -> str:
        channel = bot.get_channel(self.discord_channel_id)
        return None if channel is None else channel.name

    @property
    def discord_message_id(self) -> int:
        return self._discord_message_id

    @discord_message_id.setter
    def discord_message_id(self, value: int):
        self._discord_message_id = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoles`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoles`
            WHERE `AutoRoleId` = {id};
        """
        )

    @staticmethod
    def get_select_by_server_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoles`
            WHERE `ServerId` = {id};
        """
        )

    @staticmethod
    def get_select_by_message_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoles`
            WHERE `DiscordMessageId` = {id};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `AutoRoles` (
                `ServerId`, `DiscordChannelId`, `DiscordMessageId`
            ) VALUES (
                {self._server.id},
                {self._discord_channel_id},
                {self._discord_message_id}
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `AutoRoles`
            SET `ServerId` = {self._server.id},
            `DiscordChannelId` = {self._discord_channel_id},
            `DiscordMessageId` = {self._discord_message_id}
            WHERE `AutoRoleId` = {self._auto_role_id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `AutoRoles`
            WHERE `AutoRoleId` = {self._auto_role_id};
        """
        )
