from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.api_key_repository_abc import ApiKeyRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.api_key import ApiKey


class ApiKeyRepositoryService(ApiKeyRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        users: UserRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._users = users

        ApiKeyRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and "NULL" in value:
            return None

        return value

    def _api_key_from_result(self, sql_result: tuple) -> ApiKey:
        creator = self._get_value_from_result(sql_result[3])
        api_key = ApiKey(
            self._get_value_from_result(sql_result[1]),
            self._get_value_from_result(sql_result[2]),
            None if creator is None else self._users.get_user_by_id(int(creator)),
            self._get_value_from_result(sql_result[4]),
            self._get_value_from_result(sql_result[5]),
            id=self._get_value_from_result(sql_result[0]),
        )

        return api_key

    def get_api_keys(self) -> List[ApiKey]:
        api_keys = List(ApiKey)
        self._logger.trace(__name__, f"Send SQL command: {ApiKey.get_select_all_string()}")
        results = self._context.select(ApiKey.get_select_all_string())
        for result in results:
            api_keys.append(self._api_key_from_result(result))

        return api_keys

    def get_api_key(self, identifier: str, key: str) -> ApiKey:
        self._logger.trace(__name__, f"Send SQL command: {ApiKey.get_select_string(identifier, key)}")
        return self._api_key_from_result(self._context.select(ApiKey.get_select_string(identifier, key))[0])

    def get_api_key_by_id(self, id: int) -> ApiKey:
        self._logger.trace(__name__, f"Send SQL command: {ApiKey.get_select_by_id(id)}")
        return self._api_key_from_result(self._context.select(ApiKey.get_select_by_id(id))[0])

    def get_api_key_by_key(self, key: str) -> ApiKey:
        self._logger.trace(__name__, f"Send SQL command: {ApiKey.get_select_by_key(key)}")
        return self._api_key_from_result(self._context.select(ApiKey.get_select_by_key(key))[0])

    def add_api_key(self, api_key: ApiKey):
        self._logger.trace(__name__, f"Send SQL command: {api_key.insert_string}")
        self._context.cursor.execute(api_key.insert_string)

    def update_api_key(self, api_key: ApiKey):
        self._logger.trace(__name__, f"Send SQL command: {api_key.udpate_string}")
        self._context.cursor.execute(api_key.udpate_string)

    def delete_api_key(self, api_key: ApiKey):
        self._logger.trace(__name__, f"Send SQL command: {api_key.delete_string}")
        self._context.cursor.execute(api_key.delete_string)
