from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.model.auto_role_rule import AutoRoleRule
from bot_data.model.auto_role_rule_history import AutoRoleRuleHistory
from bot_graphql.abc.data_query_with_history_abc import DataQueryWithHistoryABC


class AutoRoleRuleQuery(DataQueryWithHistoryABC):
    def __init__(self, bot: DiscordBotServiceABC, auto_roles: AutoRoleRepositoryABC, db: DatabaseContextABC):
        DataQueryWithHistoryABC.__init__(self, "AutoRoleRule", "AutoRoleRulesHistory", AutoRoleRuleHistory, db)

        self._bot = bot
        self._auto_roles = auto_roles

        self.set_field("id", self.resolve_id)
        self.set_field("emojiName", self.resolve_emoji_name)
        self.set_field("roleId", self.resolve_role_id)
        self.set_field("roleName", self.resolve_role_name)
        self.set_field("autoRole", self.resolve_auto_role)

    @staticmethod
    def resolve_id(x: AutoRoleRule, *_):
        return x.id

    @staticmethod
    def resolve_emoji_name(x: AutoRoleRule, *_):
        return x.emoji_name

    @staticmethod
    def resolve_role_id(x: AutoRoleRule, *_):
        return x.role_id

    @staticmethod
    def resolve_role_name(x: AutoRoleRule, *_):
        return x.role_name

    def resolve_auto_role(self, x: AutoRoleRule, *_):
        return self._auto_roles.get_auto_role_by_id(x.auto_role.id)
