from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_data.abc.user_joined_game_server_repository_abc import UserJoinedGameServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.model.user import User
from bot_data.model.user_history import UserHistory
from bot_graphql.abc.data_query_with_history_abc import DataQueryWithHistoryABC
from bot_graphql.filter.user_joined_game_server_filter import UserJoinedGameServerFilter
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import UserJoinedVoiceChannelFilter
from modules.level.service.level_service import LevelService
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class UserQuery(DataQueryWithHistoryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        levels: LevelService,
        client_utils: ClientUtilsABC,
        ujs: UserJoinedServerRepositoryABC,
        ujvs: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_server: UserJoinedGameServerRepositoryABC,
        permissions: PermissionServiceABC,
    ):
        DataQueryWithHistoryABC.__init__(self, "User", "UsersHistory", UserHistory, db)

        self._bot = bot
        self._levels = levels
        self._client_utils = client_utils
        self._user_joined_game_server = user_joined_game_server
        self._ujs = ujs
        self._ujvs = ujvs
        self._permissions = permissions

        self.set_field("id", self.resolve_id)
        self.set_field("discordId", self.resolve_discord_id)
        self.set_field("name", self.resolve_name)
        self.set_field("xp", self.resolve_xp)
        self.set_field("messageCount", lambda x, *_: x.message_count)
        self.set_field("reactionCount", lambda x, *_: x.reaction_count)
        self.set_field("ontime", self.resolve_ontime)
        self.set_field("level", self.resolve_level)
        self.add_collection(
            "joinedServer",
            lambda user, *_: self._ujs.get_user_joined_servers_by_user_id(user.id),
            UserJoinedServerFilter,
        )
        self.add_collection(
            "joinedVoiceChannel",
            lambda user, *_: self._ujvs.get_user_joined_voice_channels_by_user_id(user.id),
            UserJoinedVoiceChannelFilter,
        )
        self.add_collection(
            "userJoinedGameServer",
            lambda user, *_: self._user_joined_game_server.get_user_joined_game_servers_by_user_id(user.id),
            UserJoinedGameServerFilter,
        )
        self.set_field("server", self.resolve_server)
        self.set_field("leftServer", self.resolve_left_server)

    @staticmethod
    def resolve_id(user: User, *_):
        return user.id

    @staticmethod
    def resolve_discord_id(user: User, *_):
        return user.discord_id

    @staticmethod
    def resolve_name(user: User, *_):
        return user.name

    @staticmethod
    def resolve_xp(user: User, *_):
        return user.xp

    @staticmethod
    def resolve_ontime(user: User, *_):
        return user.ontime

    @staticmethod
    def resolve_level(user: User, *_):
        return user.level

    @staticmethod
    def resolve_server(user: User, *_):
        return user.server

    @staticmethod
    def resolve_left_server(user: User, *_):
        return user.left_server
